#include "classes.h"

TrackRange::TrackRange() {
	start = 0;
	end = 0;
	max = 0;
	len = 0;
}

TrackRange::TrackRange(TrackRange* t) {
	this->start = t->start;
	this->end = t->end;
	this->max = t->max;
	this->len = t->len;
}

TrackRange::TrackRange(int start, int end, int len, int max) {
	this->start = start;
	this->end = end;
	this->max = max;
	this->len = len;
}

int TrackRange::move(int dist) {
//Pre:      dist is a positive or negative integer
//Behavior: Moves the range dist units, expanding to len if necessary
//Post:     returns overflow (under 0 or over max)
//             or 0 if range still within (0,max)

	int ret = 0;

	// adjust length
	if(end-start < len && len < max) {
		int diff = len - (end-start);

		if(dist < 0 && end == max)
			// moving towards start, at end: grow start
			start = start - diff;
		if(dist >= 0 && start == 0)
			// moving towards end, at start: grow end
			end = end + diff;
		if(dist < 0 && start == 0)
			// moving towards start, at start: shrink end
			end = end - diff;
		if(dist >= 0 && end == max)
			// moving towards end, at end: shrink start
			start = start + diff;

		if(diff > abs(dist))
			//no more to move
			dist = 0;
		else
			if(dist >= 0)
				dist = dist - diff;
			else
				dist = dist + diff;
	}
	//finally, move remaining dist
	this->start = start + dist;
	this->end = end + dist;
	
	//check for overflow signifying a leaving range
	if(start < 0){
		ret = 0 - start;
		start = 0;
	}
	if(end > max){
		ret = end - max;
		end = max;
	}

	//confine this range to max/min (for trailing start/end)
	if(end < 0){
		end = 0;
	}
	if(start > max){;
		start = max;
	}

	return ret;
}

bool TrackRange::includes(int pos) {
	return ((start <= pos) && (pos <= end));
}

bool TrackRange::includes(TrackRange* tr) {
	return this->includes(tr->start) || this->includes(tr->end) || tr->includes(this->start) || tr->includes(this->end);
}

bool TrackRange::isEmpty() {
	return start == end;
}

string TrackRange::printable() {
	ostringstream tmps;

	tmps << "(" << start << "," << end << ")";
	
	return tmps.str();
}

int TrackRange::getStart() {
	return start;
}

int TrackRange::getEnd() {
	return end;
}