#include "classes.h"

TrainCar::TrainCar() {

}

TrainCar::TrainCar(int id, int len, int dir, Train* parent) {
	this->id = id;
	this->length = len;
	this->dir = dir;
	this->speed = 0;
	this->derailed = false;
	this->distfraction = 0;
	this->speed_mult = 1;
	this->setParent(parent);
}

int TrainCar::getId() {
	return id;
}

int TrainCar::getLength() {
	return length;
}

void TrainCar::move() {
	char tmp[80];

	if(this->derailed) {
		sprintf(tmp, "tried to move derailed traincar %d", id);
		logprint(tmp);
		return;
	}

	//shortcut non-moving trains.
	if(speed == 0) return;

	//calc distance;
	double tempdist = (speed / speed_mult) + distfraction;
	int dist = (int) tempdist;
	//store fractional move for next time
	distfraction = tempdist - dist;

	if(dist == 0) return;

	//make a copy to avoid iterator dying during move
	vector<TrackSegment*> oldtracks = tracks;
	vector<TrackSegment*>::iterator p = oldtracks.begin();
	while(p != oldtracks.end()) {
		(*p)->move(this, dist);
		p++;
	}
}

void TrainCar::derail() {
	if(derailed) return;

	derailed = true;
	parent->derail();
}

void TrainCar::setSpeed(int speed) {
	this->speed = speed;
}

void TrainCar::setDirection(int dir) {
	if(dir == this->dir)
		return;

	vector<TrackSegment*>::iterator p = tracks.begin();
	while(p != tracks.end()) {
		(*p)->switchHeading(this);
		p++;
	}
}

void TrainCar::setParent(Train* p) {
	this->parent = p;
}

void TrainCar::setSpeedMult(double mult) {
	this->speed_mult = mult;
}

Train* TrainCar::getParent() {
	return this->parent;
}

bool TrainCar::isDerailed() {
	return derailed;
}

void TrainCar::addTrack(TrackSegment* track) {
	this->tracks.push_back(track);
}

void TrainCar::removeTrack(TrackSegment* track) {
	vector<TrackSegment*>::iterator p = tracks.begin();
	while(p != tracks.end()) {
		if(*p == track) {
			tracks.erase(p);
			break;
		}
		p++;
	}	
}