#include "simwin.h"
#include "configreader.h"
#include "ids.h"

#define TRACKDISPLAYFILE "configs/displaytracks.txt"
#define TURNOUTDISPLAYFILE "configs/displayturnouts.txt"
#define SENSORDISPLAYFILE "configs/displaysensors.txt"

#define TRACKSIMFILE "configs/simtracks.txt"
#define TRAINSIMFILE "configs/simtrains.txt"
#define SENSORSIMFILE "configs/simsensors.txt"
#define TURNOUTSIMFILE "configs/simturnouts.txt"

#define LOGFILE "log.txt"

int maxX, maxY;

void debugprint(string txt) {
}
void debugprint(string txt, int num) {
}

CMainWin::CMainWin() {
	Create(NULL, "Sim Gui", WS_OVERLAPPEDWINDOW, rectDefault, NULL, "TOPMENU");

	// setup memory DC
	CClientDC DC(this);
	maxX = GetSystemMetrics(SM_CXSCREEN);
	maxY = GetSystemMetrics(SM_CYSCREEN);
	m_memDC.CreateCompatibleDC(&DC);
	m_bmp.CreateCompatibleBitmap(&DC, maxX, maxY);
	m_memDC.SelectObject(&m_bmp);
	m_bkbrush.CreateStockObject(WHITE_BRUSH);
	m_memDC.SelectObject(&m_bkbrush);
	m_memDC.PatBlt(0,0, maxX, maxY, PATCOPY);

	// create drawing objects
	m_trainPen.CreatePen(PS_SOLID, 6, RGB(0,0,200));
	m_sensorOn.CreateSolidBrush(RGB(250,250,0));
	m_sensorOff.CreateSolidBrush(RGB(150,150,150));
	m_turnoutOn.CreateSolidBrush(RGB(250,0,0));
	m_turnoutOff.CreateSolidBrush(RGB(0,250,0));

	// create log box
	m_edit.Create(WS_CHILD| WS_VSCROLL | WS_BORDER| WS_VISIBLE | ES_AUTOVSCROLL | ES_MULTILINE | ES_READONLY,
		CRect(50, 400, 500, 500), this, ID_LOGBOX);

	// create save dialog
	m_savelog = new CFileDialog(FALSE, "log", 0, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, "LOG Files  (*.log)|*.log|", this);

}

//********* AppendLog
// Public method called by global logprint()
// appends txt to Logbox
//
// @@This fills up eventually, should be changed to store to 
//   a string and just display last N lines perhaps.
//
void CMainWin::AppendLog(string txt) {
	CString cs;
	cs = txt.c_str();
	m_edit.SetSel(-1,-1);
	m_edit.ReplaceSel(cs);
}

//********* getSimulator
// Public method called by DLL_API functions
// exposes internal simulator
//
Simulator* CMainWin::getSimulator() {
	return sim;
}


void CMainWin::loadSystem() {
	tracks.clear();
	sensors.clear();
	turnouts.clear();
	this->loadTracks(TRACKDISPLAYFILE);
	this->loadSensors(SENSORDISPLAYFILE);
	this->loadTurnOuts(TURNOUTDISPLAYFILE);

	logprint("display system loaded");
}

void CMainWin::loadTracks(string fname) {
	//file syntax:
	//
	// id startx starty endx endy simlength
	//
	
	TrackDisplay* tr;
	vector<int> v;
	ConfigReader cf(fname," ");

	v.clear();

	while(cf.getEntry(v) != -1) {
		if(v.size() == 6) {
			map<int,TrackDisplay*>::iterator findtr = tracks.find(v[0]);
			if(findtr == tracks.end()) {
				//create it!
				tr = new TrackDisplay(v[0]);
				tracks[tr->getId()] = tr;
			} else
				//got it!
				tr = (findtr->second);
			tr->addPart(v[1], v[2], v[3], v[4], v[5]);
		}
		v.clear();
	}
	map<int,TrackDisplay*>::iterator tri = tracks.begin();
	while(tri != tracks.end()) {
		(tri->second)->balanceSimlength();
		++tri;
	}

	logprint("created display tracks");
}

void CMainWin::loadSensors(string fname) {
	//file syntax:
	//
	// id trackid dist xoffset yoffset
	//	

	CPoint d(0,0);
	map<int,TrackDisplay*>::iterator tr;
	SensorDisplay* sn;
	vector<int> v;
	ConfigReader cf(fname," ");

	v.clear();

	while(cf.getEntry(v) != -1) {
		if(v.size() == 5) {
			//find real position
			tr = tracks.find(v[1]);
			if(tr == tracks.end()) continue; //go to next
			(tr->second)->ToAbsolute(v[2], d);
			//add offsets
			sn = new SensorDisplay(v[0], CPoint(d.x+v[3], d.y+v[4]), &m_sensorOn, &m_sensorOff);
			sensors[sn->getId()] = sn;
		}
		v.clear();
	}

	logprint("created display sensors");
}

void CMainWin::loadTurnOuts(string fname) {
	//file syntax:
	//
	// id posx posy negx negy
	//

	TurnOutDisplay* to;
	vector<int> v;
	ConfigReader cf(fname," ");

	v.clear();

	while(cf.getEntry(v) != -1) {
		if(v.size() == 5) {
			to = new TurnOutDisplay(v[0], CPoint(v[1],v[2]), CPoint(v[3],v[4]), &m_turnoutOn, &m_turnoutOff);
			turnouts[to->getId()] = to;
		}
		v.clear();
	}

	logprint("created display turnouts");

}

//********* draw System
// Internal method. updates display state for all objects
//
void CMainWin::drawSystem() {
	CPen* origpen = m_memDC.GetCurrentPen();

	map<int,TrackRange*> pos;
	map<int,TrackRange*>::iterator it;
	TrackDisplay* track;
	CPoint pnt;

	//draw tracks and cars on them
	map<int,TrackDisplay*>::iterator p = tracks.begin();
	while(p != this->tracks.end()) {
		track = (p->second);
		track->Draw(m_memDC);

		//draw cars
		m_memDC.SelectObject(&m_trainPen);

		sim->getCarPositions(track->getId(), pos);
		it = pos.begin();
		while(it != pos.end()) {
			track->ToAbsolute((it->second)->getStart(),pnt);
			m_memDC.MoveTo(pnt);
			track->ToAbsolute((it->second)->getEnd(),pnt);
			m_memDC.LineTo(pnt);
			it++;
		}
		m_memDC.SelectObject(origpen);
		p++;
	}

	//update and draw sensors
	map<int,SensorDisplay*>::iterator s = sensors.begin();
	while(s != this->sensors.end()) {
		(s->second)->setStatus(
			sim->Get_Sensor_Status(
				(short)(s->second)->getId()
			)
		);
		(s->second)->Draw(m_memDC);
		s++;
	}

	//update and draw turnouts
	map<int,TurnOutDisplay*>::iterator t = turnouts.begin();
	while(t != this->turnouts.end()) {
		(t->second)->setStatus(
			sim->Get_TO_Status(
				(short)(t->second)->getId()
			)
		);
		(t->second)->Draw(m_memDC);
		t++;
	}

}

//**** CAPP stuff

CApp::InitInstance() {
	m_pMainWnd = new CMainWin;
	m_pMainWnd->ShowWindow(m_nCmdShow);
	m_pMainWnd->UpdateWindow();

	return TRUE;
}


//***** MESSAGE HANDLING

BEGIN_MESSAGE_MAP(CMainWin, CFrameWnd)
	ON_WM_TIMER()
	ON_WM_PAINT()
	ON_WM_DESTROY()
	ON_COMMAND(IDM_LOAD_SYSTEM, OnLoadSystem)
	ON_COMMAND(IDM_SAVE_LOG, OnSaveLog)
	ON_COMMAND(IDM_START_SIM, OnStartSimulation)
	ON_COMMAND(IDM_STOP_SIM, OnStopSimulation)
	ON_COMMAND(IDM_EXIT, OnExit)

	ON_COMMAND(IDM_TOGGLE_T0, OnToggleT0)
	ON_COMMAND(IDM_TOGGLE_T1, OnToggleT1)
	ON_COMMAND(IDM_TOGGLE_T2, OnToggleT2)
	ON_COMMAND(IDM_TOGGLE_T3, OnToggleT3)
	ON_COMMAND(IDM_TOGGLE_T4, OnToggleT4)
	ON_COMMAND(IDM_TOGGLE_T5, OnToggleT5)

	ON_COMMAND(IDM_TRAIN1_S1, OnTrain1S1)
	ON_COMMAND(IDM_TRAIN1_S2, OnTrain1S2)
	ON_COMMAND(IDM_TRAIN1_S3, OnTrain1S3)
	ON_COMMAND(IDM_TRAIN2_S1, OnTrain2S1)
	ON_COMMAND(IDM_TRAIN2_S2, OnTrain2S2)
	ON_COMMAND(IDM_TRAIN2_S3, OnTrain2S3)

END_MESSAGE_MAP()

afx_msg void CMainWin::OnPaint() {

	CPaintDC DC(this);
	DC.BitBlt(0,0,maxX, maxY, &m_memDC, 0, 0, SRCCOPY);
}

afx_msg void CMainWin::OnExit() {
	SendMessage(WM_CLOSE);
}

afx_msg void CMainWin::OnDestroy() {
	KillTimer(IDT_MAIN_SIM);
}

afx_msg void CMainWin::OnTimer(UINT ID) {
	if(ID == IDT_MAIN_SIM) {
		// clear memory screen
		m_memDC.SelectObject(&m_bkbrush);
		m_memDC.PatBlt(0,0, maxX, maxY, PATCOPY);
		//run and draw step
		sim->step();
		this->drawSystem();
		//invalidate area above log box
		InvalidateRect(CRect(0,0,800,345));
	}
}

afx_msg void CMainWin::OnLoadSystem() {

	// clear log
	m_edit.SetWindowText("");

	// clear memory screen
	m_memDC.SelectObject(&m_bkbrush);
	m_memDC.PatBlt(0,0, maxX, maxY, PATCOPY);

	sim = new Simulator();

	sim->loadTracks(TRACKSIMFILE);
	sim->loadSensors(SENSORSIMFILE);
	sim->loadTrains(TRAINSIMFILE);
	sim->loadTurnOuts(TURNOUTSIMFILE);

	//sim->motor(1, 0, 5);
	//sim->motor(2, 1, 10);

	this->loadSystem();

	this->drawSystem();
	
	InvalidateRect(CRect(0,0,800,345));
}

afx_msg void CMainWin::OnSaveLog() {
	CString tmp;
	string tmps;
	string filename;
	ofstream fd;

	if(m_savelog->DoModal() == IDOK) {
		filename = m_savelog->GetPathName();
		fd.open(filename.c_str());
		m_edit.GetWindowText(tmp);
		tmps = tmp;
		fd << tmps;
		fd.close();
	} else {
		AfxMessageBox("Error Saving File");
	}
}

afx_msg void CMainWin::OnStartSimulation() {

	if(this->SetTimer(IDT_MAIN_SIM, 100, NULL) != IDT_MAIN_SIM) {
		AfxMessageBox("Failed to start simulation timer");
		this->PostMessage(WM_DESTROY);
	}
}

afx_msg void CMainWin::OnStopSimulation() {
	KillTimer(IDT_MAIN_SIM);
}






//*** EXTERNAL TEST

afx_msg void CMainWin::OnToggleT0() {
	TO_Toggle(0);
	AfxMessageBox("Toggled T0");
}
afx_msg void CMainWin::OnToggleT1(){
	TO_Toggle(1);
	AfxMessageBox("Toggled T1");
}
afx_msg void CMainWin::OnToggleT2(){
	TO_Toggle(2);
	AfxMessageBox("Toggled T2");
}
afx_msg void CMainWin::OnToggleT3(){
	TO_Toggle(3);
	AfxMessageBox("Toggled T3");
}
afx_msg void CMainWin::OnToggleT4(){
	TO_Toggle(4);
	AfxMessageBox("Toggled T4");
}
afx_msg void CMainWin::OnToggleT5(){
	TO_Toggle(5);
	AfxMessageBox("Toggled T5");
}

afx_msg void CMainWin::OnTrain1S1(){
	DCC_Estop(0,1,0);
	AfxMessageBox("Train 1 EStop");
}
afx_msg void CMainWin::OnTrain1S2(){
	DCC_motor(0, 1, 0, 2);
	AfxMessageBox("Train 1 Speed 2 dir 0");
}
afx_msg void CMainWin::OnTrain1S3(){
	DCC_motor(0, 1, 0, 8);
	AfxMessageBox("Train 1 Speed 8 dir 0");
}
afx_msg void CMainWin::OnTrain2S1(){
	DCC_motor(0, 1, 1, 2);
	AfxMessageBox("Train 1 Speed 2 dir 1");
}
afx_msg void CMainWin::OnTrain2S2(){
	DCC_motor(0, 2, 0, 5);
	AfxMessageBox("Train 2 Speed 5 dir 0");
}
afx_msg void CMainWin::OnTrain2S3(){
	DCC_motor(0, 2, 1, 5);
	AfxMessageBox("Train 2 Speed 5 dir 1");
}

//***** LET'S GO

CApp TheApp;

void logprint(string txt,bool newline) {
	if(newline)
		txt = "\r\n" + txt;
	CMainWin* w = (CMainWin*) TheApp.GetMainWnd();
	w->AppendLog(txt);
}
