#include <math.h>
#include "tracksdisplay.h"

#define DISP_DOTSIZE 7

TrackDisplay::TrackDisplay() {
}

TrackDisplay::TrackDisplay(int id) {
	this->id = id;
}

void TrackDisplay::addPart(int startx, int starty, int endx, int endy, int simlength) {
	TrackDisplayPart* p = new TrackDisplayPart();
	p->start = CPoint(startx, starty);
	p->end = CPoint(endx, endy);
	p->simlength = simlength;

	parts.push_back(p);
}

void TrackDisplay::balanceSimlength() {
	//get total simlength and reallength
	int simlength = 0;
	double reallength = 0;
	vector<TrackDisplayPart*>::iterator part = parts.begin();
	while(part != parts.end()) {
		simlength += (*part)->simlength;
		reallength += abs(this->PDistance((*part)->start, (*part)->end));
		++part;
	}
	//now balance ratios
	double ratio;
	TrackDisplayPart* tempart;
	int newsimlength = 0;
	part = parts.begin();
	while(part != parts.end()) {
		tempart = *part;
		ratio = abs(this->PDistance(tempart->start, tempart->end)) / reallength;
		tempart->simlength = (simlength * ratio)+0.5;
		newsimlength += tempart->simlength;
		++part;
	}
	//give any error to the first one
	int diff = simlength - newsimlength;
	part = parts.begin();
	if(part != parts.end()) {
		(*part)->simlength += diff;
	}

}

void TrackDisplay::Draw(CDC &dc) {
	vector<TrackDisplayPart*>::iterator part;
	part = parts.begin();
	while(part != parts.end()) {
		dc.MoveTo((*part)->start);
		dc.LineTo((*part)->end);
		++part;
	}
}

void TrackDisplay::ToAbsolute(int dist, CPoint &pnt) {

	//find correct part
	CPoint st, en;
	int sm = -1;
	int sofar = 0;
	int prevpartslen = 0;
	vector<TrackDisplayPart*>::iterator part;
	part = parts.begin();
	while(part != parts.end()) {
		prevpartslen = sofar;
		sofar += (*part)->simlength;
		if(sofar >= dist) {
			st = (*part)->start;
			en = (*part)->end;
			sm = (*part)->simlength;
			break;
		}
		++part;
	}

	dist -= prevpartslen;

	if(sm == -1) {
		//distance too long!
		return;
	}

	//double length = PDistance(start, end);
	int xlen = st.x - en.x;
	int ylen = st.y - en.y;

	//double lenratio = length/simlength;
	double ratio = dist/(double)sm;

	pnt = CPoint((int)st.x-xlen*ratio, (int)st.y-ylen*ratio);
	return;

}

int TrackDisplay::getId() {
	return id;
}

double TrackDisplay::PDistance(CPoint a, CPoint b) {
	return sqrt(pow((a.x-b.x),2)+pow((a.y-b.y),2));
}

//SensorDisplay

SensorDisplay::SensorDisplay(int id, CPoint loc, CBrush* onBrush, CBrush* offBrush) {
	this->id = id;
	this->status = false;
	this->loc = loc;
	this->onBrush = onBrush;
	this->offBrush = offBrush;
}

void SensorDisplay::setStatus(bool status) {
	this->status = status;
}

void SensorDisplay::Draw(CDC &dc) {
	CBrush* savedbrush = dc.GetCurrentBrush();
	CPen* savedpen = dc.GetCurrentPen();
	CPen n;
	n.CreateStockObject(NULL_PEN);

	if(status == true)
		dc.SelectObject(onBrush);
	else {
		dc.SelectObject(&n);
		dc.SelectObject(offBrush);
	}

	dc.Rectangle(loc.x,loc.y,loc.x+DISP_DOTSIZE,loc.y+DISP_DOTSIZE);
	
	//char t[5];
	//itoa(id, t, 10);
	//dc.TextOut(loc.x, loc.y, t);

	dc.SelectObject(savedpen);
	dc.SelectObject(savedbrush);
}

int SensorDisplay::getId() {
	return id;
}


//TurnOutDisplay

TurnOutDisplay::TurnOutDisplay(int id, CPoint loc, CPoint loc2, CBrush* onBrush, CBrush* offBrush){
	this->id = id;
	this->status = false;
	this->loc = loc;
	this->loc2 =  loc2;
	this->onBrush = onBrush;
	this->offBrush = offBrush;
}

void TurnOutDisplay::setStatus(bool status){
	this->status = status;
}

void TurnOutDisplay::Draw(CDC &dc) {
	CBrush* savedbrush = dc.GetCurrentBrush();

	dc.SelectObject(onBrush);
	if(status == false)
		dc.Ellipse(loc.x,loc.y,loc.x+DISP_DOTSIZE,loc.y+DISP_DOTSIZE);
	else
		dc.Ellipse(loc2.x,loc2.y,loc2.x+DISP_DOTSIZE,loc2.y+DISP_DOTSIZE);

	dc.SelectObject(offBrush);
	if(status == false)
		dc.Ellipse(loc2.x,loc2.y,loc2.x+DISP_DOTSIZE,loc2.y+DISP_DOTSIZE);
	else
		dc.Ellipse(loc.x,loc.y,loc.x+DISP_DOTSIZE,loc.y+DISP_DOTSIZE);

	dc.SelectObject(savedbrush);
}

int TurnOutDisplay::getId() {
	return id;
}